#pragma once

#include <list>

namespace tipbase
{

class IPCPacket
{
public:
	IPCPacket()
	{
		size = 0;
		data = NULL;
	}

	void Release()
	{
		if( NULL != data )
		{
			delete [] data;
			data = NULL;
		}
	}

	int		size;
	BYTE*	data;
};

class IIPCKey
{
public:
	virtual ~IIPCKey()
	{

	}

	virtual void *GetData() = 0;
	virtual int GetSize() = 0;
};


class IIPCHandle;
class IIPCReceiveCallback
{
public:
	virtual ~IIPCReceiveCallback()
	{

	}
	virtual int Callback(IPCPacket& packet, IIPCHandle *callback_handle ) = 0;
};

class IIPCHandle
{
public:
	IIPCHandle()
	{
		m_callback = NULL;
	}

	virtual ~IIPCHandle()
	{
		m_callback = NULL;
	}

	virtual void SetReadCallback( IIPCReceiveCallback *callback )
	{
		m_callback = callback;
	}
	
	virtual bool IsValid()
	{
		return true;
	}
	virtual bool Read(IPCPacket& packet) = 0;
	virtual bool Write( const BYTE *data, int size ) = 0;
	virtual IIPCKey* GetKey() = 0;
	virtual void Close() = 0;
protected:
	IIPCReceiveCallback	*m_callback;
};

typedef std::list< IIPCHandle * > IIPCHandleList;
class IIPCServer
{
public:
	virtual ~IIPCServer()
	{

	}

	virtual bool Create( IIPCKey *host ) = 0;
	virtual IIPCHandle *Find( IIPCKey *client, IIPCReceiveCallback *callback = NULL ) = 0;
	virtual IIPCHandle *Accept()
	{
		return NULL;
	}
	virtual IIPCHandleList GetHandleList() = 0;
};
};